#ifndef ALLCOMMANDS_H
#define ALLCOMMANDS_H

#include "../MyRobot.h"
#include "WPILib.h"
#include "../Subsystems/Drive.h"
#include "../Subsystems/Elevator.h"

// ******************** DRIVE Commands *********************

class DriveAutoDrive: public Command {
public:
	DriveAutoDrive(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder,
				   bool useBrake, float angle, Drive::AngleFrom angleFrom, double target,
				   float driveMax, float driveRamp);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*               m_cancel;
	double              m_timeout;
	bool				m_resetEncoder;
	Drive::DriveEncoder m_useEncoder;
	bool                m_useBrake;
	float				m_angle;
	Drive::AngleFrom	m_angleFrom;
	double				m_target;
	float				m_driveMax;
	float				m_driveRamp;
};

class DriveAutoDriveElev: public Command {
public:
	DriveAutoDriveElev(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder,
					   bool useBrake, float angle, Drive::AngleFrom angleFrom, double target,
					   float driveMax, float driveRamp, double trigger, Elevator::ElevPosition position);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*              		m_cancel;
	double              	m_timeout;
	bool					m_resetEncoder;
	Drive::DriveEncoder 	m_useEncoder;
	bool               	 	m_useBrake;
	float					m_angle;
	Drive::AngleFrom		m_angleFrom;
	double					m_target;
	float					m_driveMax;
	double              	m_trigger;
	Elevator::ElevPosition	m_elevPosition;
	bool					m_atTrigger;
	float					m_driveRamp;
};

class DriveAutoDriveStrafe: public Command {
public:
	DriveAutoDriveStrafe(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder,
						 bool useBrake, float angle, Drive::AngleFrom angleFrom, double target,
						 float driveMax, float driveRamp, float strafeMax);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*          			m_cancel;
	double            		m_timeout;
	bool					m_resetEncoder;
	Drive::DriveEncoder 	m_useEncoder;
	bool                	m_useBrake;
	float					m_angle;
	Drive::AngleFrom		m_angleFrom;
	double					m_target;
	float					m_driveMax;
	float               	m_strafeMax;
	float					m_driveRamp;
};

class DriveAutoDriveTurn: public Command {
public:
	DriveAutoDriveTurn(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder,
					   bool useBrake, float angle, Drive::AngleFrom angleFrom, double target,
					   float driveMax, float driveRamp, float rotateMax);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*               	m_cancel;
	double              	m_timeout;
	bool					m_resetEncoder;
	Drive::DriveEncoder 	m_useEncoder;
	bool                	m_useBrake;
	float					m_angle;
	Drive::AngleFrom		m_angleFrom;
	double					m_target;
	float               	m_driveMax;
	float					m_rotateMax;
	float					m_driveRamp;
};

class DriveAutoRotate: public Command {
public:
	DriveAutoRotate(bool* cancel, double timeout, float angle, Drive::AngleFrom angleFrom, float rotateMax);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*          		m_cancel;
	double            	m_timeout;
	float				m_angle;
	Drive::AngleFrom	m_angleFrom;
	float				m_rotateMax;
};

class DriveAutoStrafe: public Command {
public:
	DriveAutoStrafe(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder,
					bool useBrake, float angle, Drive::AngleFrom angleFrom, double target,
					float strafeMax);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	bool*               m_cancel;
	double              m_timeout;
	bool				m_resetEncoder;
	Drive::DriveEncoder m_useEncoder;
	bool                m_useBrake;
	float				m_angle;
	Drive::AngleFrom	m_angleFrom;
	double				m_target;
	float				m_strafeMax;
};

class DriveResetEncoder: public Command {
public:
	DriveResetEncoder();
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
};

class DriveSetFeederAngle: public Command {
public:
	DriveSetFeederAngle(bool useSonar);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	bool m_useSonar;
	bool m_logReset;
};

class DriveSetMarkAngle: public Command {
public:
	DriveSetMarkAngle();
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
};

class DriveSetMode: public Command {
public:
	DriveSetMode(Drive::DriveMode mode);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Drive::DriveMode m_mode;
};

// ******************* ELEVATOR Commands *******************

class ElevAutoCenter: public Command {
public:
	ElevAutoCenter();
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
};

class ElevCanGrab: public Command {
public:
	ElevCanGrab(Elevator::SolenoidState state, double delay);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Elevator::SolenoidState m_state;
	double m_delay;
};

class ElevIncrement: public Command {
public:
	ElevIncrement(Elevator::ElevDirection direction);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Elevator::ElevDirection m_direction;
};

class ElevSetPosition: public Command {
public:
	ElevSetPosition(Elevator::ElevPosition position);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();

private:
	Elevator::ElevPosition m_position;
};

class ElevSetStackHold: public Command {
public:
	ElevSetStackHold(Elevator::SolenoidState state);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Elevator::SolenoidState m_state;
};

class ElevSetToteCenter: public Command {
public:
	ElevSetToteCenter(Elevator::SolenoidState state);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Elevator::SolenoidState m_state;
};

class ElevSetToteEject: public Command {
public:
	ElevSetToteEject(Elevator::SolenoidState state);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	Elevator::SolenoidState m_state;
};

class ElevWaitForDistance: public Command {
public:
	ElevWaitForDistance(double distance);
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
private:
	double m_distance;
};

class ElevWaitForOnTarget: public Command {
public:
	ElevWaitForOnTarget();
	void Initialize();
	void Execute();
	bool IsFinished();
	void End();
	void Interrupted();
};

// *************** AUTONOMOUS Command Groups ***************

class Auto3ToteToZone: public CommandGroup {
public:
	Auto3ToteToZone(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoBinToFeederL: public CommandGroup {
public:
	AutoBinToFeederL(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};
class AutoBinToFeederR: public CommandGroup {
public:
	AutoBinToFeederR(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoBinToWall: public CommandGroup {
public:
	AutoBinToWall(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoBinToWallRobotToZone: public CommandGroup {
public:
	AutoBinToWallRobotToZone(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoBinToZoneL: public CommandGroup {
public:
	AutoBinToZoneL(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoBinToZoneR: public CommandGroup {
public:
	AutoBinToZoneR(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoRobotToZone: public CommandGroup {
public:
	AutoRobotToZone(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoPickupTote: public CommandGroup {
public:
	AutoPickupTote();
};

class AutoStepBinL: public CommandGroup {
public:
	AutoStepBinL(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoStepBinR: public CommandGroup {
public:
	AutoStepBinR(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoToteBinToZoneL: public CommandGroup {
public:
	AutoToteBinToZoneL(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoToteBinToZoneR: public CommandGroup {
public:
	AutoToteBinToZoneR(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

class AutoToteToZone: public CommandGroup {
public:
	AutoToteToZone(double wait);
	bool IsFinished();
private:
	bool m_cancel;

};

class AutoSolo3ToteToZone: public CommandGroup {
public:
	AutoSolo3ToteToZone(double wait);
	bool IsFinished();
private:
	bool m_cancel;
};

#endif
