#include "AllCommands.h"

DriveAutoDriveElev::DriveAutoDriveElev(bool* cancel, double timeout, bool resetEncoder, Drive::DriveEncoder useEncoder, bool useBrake,
									   float angle, Drive::AngleFrom angleFrom, double target, float driveMax, float driveRamp,
									   double trigger, Elevator::ElevPosition position) {
	Requires(MyRobot::drive);

	m_cancel = cancel;
	m_timeout = timeout;
	m_resetEncoder = resetEncoder;
	m_useEncoder = useEncoder;
	m_useBrake = useBrake;
	m_angle = angle;
	m_angleFrom = angleFrom;
	m_target = target;
	m_driveMax = driveMax;
	m_trigger = trigger;
	m_elevPosition = position;
	m_atTrigger = false;
	m_driveRamp = driveRamp;
}

void DriveAutoDriveElev::Initialize() {
	if (m_timeout > 0) SetTimeout(m_timeout);
	MyRobot::drive->InitAutoDrive(m_resetEncoder, m_useEncoder, m_useBrake, m_angle, m_angleFrom, m_target, m_trigger, m_driveMax, m_driveRamp);
	m_atTrigger = false;
}

void DriveAutoDriveElev::Execute() {
	if (!m_atTrigger) {
		if (MyRobot::drive->IsAtTrigger()) {
			m_atTrigger = true;
			MyRobot::elevator->SetElevPosition(m_elevPosition);
		}
	}
}

bool DriveAutoDriveElev::IsFinished() {
	if (IsTimedOut()) {
		MyRobot::robotLog->Write("Drive:    Auto Drive Timeout");
		*m_cancel = true;
		return true;
	}

	return MyRobot::drive->IsOnTarget();
}

void DriveAutoDriveElev::End() {
	MyRobot::drive->CancelCommand();
}

void DriveAutoDriveElev::Interrupted() {
	MyRobot::drive->CancelCommand();
	MyRobot::robotLog->Write("Command Interrupt: DriveAutoDriveElev");
}
