#ifndef Drive_H
#define Drive_H

#include "Commands/Subsystem.h"
#include "WPILib.h"
#include "PIDControl.h"
#include "../Robotmap.h"

class Drive: public Subsystem {
public:
	typedef enum {fStart, fNewMark, fMark, fFeeder} AngleFrom;
	typedef enum {aNone, aDrive, aStrafe, aRotate, aDriveTurn, aDriveStrafe} AutoMode;
	typedef enum {eBoth, eLeft, eRight} DriveEncoder;
	typedef enum {mDriveRotate, mDriveStrafe, mStrafe, mFeederDrive} DriveMode;
	typedef enum {wAll, wRear, wFront} DriveWheels;

	Drive();
	~Drive();
	void        CancelCommand();
	void		Execute(bool showPID);
	double		GetEncoderDistance(DriveEncoder encoder);
	float       GetGyroAngle();
	void		InitAutoDrive      (bool resetEncoder, DriveEncoder useEncoder, bool useBrake, float angle, AngleFrom angleFrom,
									double target, double trigger, float driveMax, float driveRamp);
	void		InitAutoDriveStrafe(bool resetEncoder, DriveEncoder useEncoder,bool useBrake, float angle, AngleFrom angleFrom,
									double target, double trigger, float driveMax, float driveRamp, float strafeMax);
	void 		InitAutoDriveTurn  (bool resetEncoder, DriveEncoder useEncoder,bool useBrake, float angle, AngleFrom angleFrom,
									double target, double trigger, float driveMax, float driveRamp, float rotateMax);
	void 		InitAutoRotate     (float angle, AngleFrom angleFrom, float rotateMax);
	void		InitAutoStrafe     (bool resetEncoder, DriveEncoder useEncoder,bool useBrake, float angle, AngleFrom angleFrom,
									double target, double trigger, float strafeMax);

	void 		InitDefaultCommand();
	bool		IsOnTarget();
	bool		IsAtTrigger();
	void        ResetEncoders();
	void		SetConstant(std::string key, int32_t value);
	void		SetDriveMode(DriveMode mode);
	void        SetDrivePID(float dThreshold);
	bool        SetFeederAngle(bool useSonar, bool logReset);
	void		SetMarkAngle();
	void		SetSonarEnable(bool enable);
	void  		SetRotatePID();
	void		TuneDrivePID();
	void        TuneRotatePID();

private:
	Encoder*	m_leftEncoder;
	Encoder*	m_rightEncoder;
	Gyro*		m_gyro;
	PIDControl*	m_drivePID;
	PIDControl*	m_rotatePID;
	Ultrasonic* m_leftSonar;
	Ultrasonic* m_rightSonar;

	#if (MY_ROBOT == 0)							// Schumacher
		VictorSP*		m_motorLF;
		VictorSP*		m_motorLR;
		VictorSP*		m_motorRF;
		VictorSP*		m_motorRR;
	#else										// Otis
		Victor*		m_motorLF;
		Victor*		m_motorLR;
		Victor*		m_motorRF;
		Victor*		m_motorRR;
	#endif

	bool			m_atTrigger;
	bool			m_onTarget;
	bool			m_rampDone;

	char			m_log[100];

	int				m_autoCounter;
	AutoMode		m_autoMode;
	float			m_angleMark;
	float			m_angleFeeder;
	float			m_angleTarget;
	double			m_distLast;
	double			m_distTarget;
	double			m_distTrigger;
	int				m_doneCounter;
	DriveMode		m_driveMode;
	float			m_maxDrive;
	float       	m_maxRotate;
	float     	  	m_maxStrafe;
	float			m_pwmDrive;
	float   	    m_pwmRotate;
	float			m_pwmStrafe;
	float			m_rampDrive;
	DriveEncoder	m_useEncoder;

	double		    f_reverseMaxRate;

	void		ArcadeDrive(float drive, float rotate);
	float 		DrivePWM(float curPWM, float pidPWM);
	void		InitAuto	       (bool resetEncoder, DriveEncoder useEncoder, bool useBrake, float angle, AngleFrom angleFrom,
									double target, double trigger);
	float		Limit(float Value);
	void 		MecanumDrive(float drive, float strafe, float rotate, DriveWheels driveWheels);
	float       RotateError(float target);
};

#endif
